<?php

/**
 * @file
 * Contains \Drupal\messagebird\MessagebirdService.
 */

namespace Drupal\messagebird;

use MessageBird\Client;
use MessageBird\Exceptions\AuthenticateException;
use MessageBird\Exceptions\BalanceException;
use MessageBird\Objects\Balance;
use MessageBird\Objects\Message;

/**
 * Class MessagebirdService
 * @package Drupal\messagebird
 */
class MessagebirdService {

  /**
   * Send a SMS message through MessageBird.
   *
   * @param $body
   * @param $recipients
   * @param null $originator
   * @param array $options
   * @return Message|bool
   */
  public function sendMessage($body, $recipients, $originator = NULL, $options = array()) {
    $config = \Drupal::config('messagebird.settings');

    if (!is_array($recipients)) {
      $recipients = array($recipients);
    }

    if (empty($originator)) {
      $originator = $config->get('default.originator');
    }

    if ($config->get('debug.mode')) {
      $debug_params = array(
        'Body' => $body,
        'Recipients' => implode(', ', $recipients),
        'Originator' => $originator,
      );
      foreach ($debug_params as $key => $value) {
        drupal_set_message(t('@messagebird_debug_key: @messagebird_debug_value', array(
          '@messagebird_debug_key' => $key,
          '@messagebird_debug_value' => $value,
        )));
      }
    }

    try {
      $messagebird = $this->getClient();

      $message = new Message();
      $message->originator = $originator;
      $message->recipients = $recipients;
      $message->body = $body;

      // Append the optional parameters.
      foreach ($options as $parameter => $value) {
        if (!empty($parameter) && !empty($value)) {
          $message->{$parameter} = $value;
        }
      }

      // Send the message to the recipient(s).
      $response = $messagebird->messages->create($message);
    }
    catch (AuthenticateException $e) {
      $message = 'Message sending failed due to an authentication exception.';
      // Authentication failed. Is this a wrong access_key?
      \Drupal::logger('messagebird')->error($message);
    }
    catch (BalanceException $e) {
      // That means that you are out of credits. Only called on creation of a object.
      $message = 'Message sending failed due to a shortage of credits.';
      \Drupal::logger('messagebird')->error($message);
    }
    catch (\Exception $e) {
      // Request failed. More information can be found in the body.
      $message = 'Message sending failed due to an unexpected exception: @error';
      \Drupal::logger('messagebird')->error($message, array(
        '@error' => $e->getMessage(),
      ));
    }
    finally {
      return (isset($response)) ? $response : FALSE;
    }
  }

  /**
   * Retrieve the MessageBird account balance.
   *
   * @return Balance|bool
   */
  public function getBalance() {
    try {
      $messagebird = $this->getClient();
      $balance = $messagebird->balance->read();
    }
    catch (AuthenticateException $e) {
      // Authentication failed. Is this a wrong access_key?
      $message = 'Retrieving the account balance failed due to an authentication exception.';
      \Drupal::logger('messagebird')->error($message);
    }
    catch (\Exception $e) {
      // Request failed. More information can be found in the body.
      $message = 'Retrieving the account balance failed due to an unexpected exception: @error';
      \Drupal::logger('messagebird')->error($message, array(
        '@error' => $e->getMessage(),
      ));
    }
    finally {
      return (isset($balance)) ? $balance : FALSE;
    }
  }

  /**
   * Helper for getting a MessageBird Client object.
   */
  private function getClient() {
    $api_key = \Drupal::config('messagebird.settings')->get('api.key');
    $client = new Client($api_key);

    return $client;
  }
}